`timescale 1ns/1ps

module h_compact
#(
	parameter DATA_WIDTH = 32
)
(
    input      [DATA_WIDTH-1:0] data_i         ,
    input      [DATA_WIDTH-1:0] key_schedule0_i,
    input      [DATA_WIDTH-1:0] key_schedule1_i,
	
    output     [DATA_WIDTH-1:0] data_o
);

reg [0:(DATA_WIDTH*8)-1][(DATA_WIDTH/4)-1:0] mulBy5B_memory   ;
reg [0:(DATA_WIDTH*8)-1][(DATA_WIDTH/4)-1:0] mulByEF_memory   ;
reg [0:(DATA_WIDTH*8)-1][(DATA_WIDTH/4)-1:0] permut_memory_q0 ;
reg [0:(DATA_WIDTH*8)-1][(DATA_WIDTH/4)-1:0] permut_memory_q1 ;

memory_5B
#(
    .WIDTH(DATA_WIDTH/4),
    .DEPTH(DATA_WIDTH*8)
)
memory_5BINST
(
    .mulBy5B_memory_o(mulBy5B_memory)
);

memory_EF
#(
    .WIDTH(DATA_WIDTH/4),
    .DEPTH(DATA_WIDTH*8)
)
memory_EFINST
(
    .mulByEF_memory_o(mulByEF_memory)
);

memory_q0
#(
    .WIDTH(DATA_WIDTH/4),
    .DEPTH(DATA_WIDTH*8)
)
memory_q0INST
(
    .permut_memory_q0_o(permut_memory_q0)
);

memory_q1
#(
    .WIDTH(DATA_WIDTH/4),
    .DEPTH(DATA_WIDTH*8)
)
memory_q1INST
(
    .permut_memory_q1_o(permut_memory_q1)
);


wire [(DATA_WIDTH/4)-1:0] data_upmost ;
wire [(DATA_WIDTH/4)-1:0] data_upper  ;  
wire [(DATA_WIDTH/4)-1:0] data_lower  ;  
wire [(DATA_WIDTH/4)-1:0] data_lowmost;

wire [(DATA_WIDTH/4)-1:0] key_schedule0_upmost ;
wire [(DATA_WIDTH/4)-1:0] key_schedule0_upper  ;
wire [(DATA_WIDTH/4)-1:0] key_schedule0_lower  ;
wire [(DATA_WIDTH/4)-1:0] key_schedule0_lowmost;

wire [(DATA_WIDTH/4)-1:0] key_schedule1_upmost ;
wire [(DATA_WIDTH/4)-1:0] key_schedule1_upper  ;
wire [(DATA_WIDTH/4)-1:0] key_schedule1_lower  ;
wire [(DATA_WIDTH/4)-1:0] key_schedule1_lowmost;

wire [(DATA_WIDTH/4)-1:0] s0_yield;
wire [(DATA_WIDTH/4)-1:0] s1_yield;
wire [(DATA_WIDTH/4)-1:0] s2_yield;
wire [(DATA_WIDTH/4)-1:0] s3_yield;

wire [(DATA_WIDTH/4)-1:0] z0;
wire [(DATA_WIDTH/4)-1:0] z1;
wire [(DATA_WIDTH/4)-1:0] z2;
wire [(DATA_WIDTH/4)-1:0] z3;

assign {data_lowmost,data_lower,data_upper,data_upmost} = data_i[(DATA_WIDTH-1):0];

assign {key_schedule0_lowmost,key_schedule0_lower,key_schedule0_upper,key_schedule0_upmost} = key_schedule0_i[(DATA_WIDTH-1):0];
assign {key_schedule1_lowmost,key_schedule1_lower,key_schedule1_upper,key_schedule1_upmost} = key_schedule1_i[(DATA_WIDTH-1):0];

assign s0_yield = permut_memory_q1[permut_memory_q0[permut_memory_q0[data_upmost]  ^ key_schedule1_upmost] ^ key_schedule0_upmost];
assign s1_yield = permut_memory_q0[permut_memory_q0[permut_memory_q1[data_upper]   ^ key_schedule1_upper] ^ key_schedule0_upper];
assign s2_yield = permut_memory_q1[permut_memory_q1[permut_memory_q0[data_lower]   ^ key_schedule1_lower] ^ key_schedule0_lower];
assign s3_yield = permut_memory_q0[permut_memory_q1[permut_memory_q1[data_lowmost] ^ key_schedule1_lowmost] ^ key_schedule0_lowmost];

assign z0 = s0_yield 				 ^ mulByEF_memory[s1_yield] ^ mulBy5B_memory[s2_yield] ^ mulBy5B_memory[s3_yield];
assign z1 = mulBy5B_memory[s0_yield] ^ mulByEF_memory[s1_yield] ^ mulByEF_memory[s2_yield] ^ s3_yield				 ;
assign z2 = mulByEF_memory[s0_yield] ^ mulBy5B_memory[s1_yield] ^ s2_yield				   ^ mulByEF_memory[s3_yield];
assign z3 = mulByEF_memory[s0_yield] ^ s1_yield					^ mulByEF_memory[s2_yield] ^ mulBy5B_memory[s3_yield];

assign data_o = {z3,z2,z1,z0};

endmodule
