module s_boxTB();

parameter DATA_WIDTH = 8;
parameter Q_LEFT     = 0;
parameter Q_MIDDLE   = 0;
parameter Q_RIGHT    = 0;

reg 				  clk_i	     ;
reg 				  reset_ni   ;
reg [DATA_WIDTH-1:0]  left_q_i   ;
reg [DATA_WIDTH-1:0]  middle_q_i ;
reg [DATA_WIDTH-1:0]  right_q_i  ;

wire [DATA_WIDTH-1:0] left_q_o   ;
wire [DATA_WIDTH-1:0] middle_q_o ;
wire [DATA_WIDTH-1:0] right_q_o  ;

s_box
#(
	.DATA_WIDTH(DATA_WIDTH),
	.Q_LEFT    (0	  	  ),  
	.Q_MIDDLE  (0     	  ),
	.Q_RIGHT   (1     	  )
)
sbox1
(
	.clk_i	    (clk_i	    ),    
	.reset_ni   (reset_ni   ),
	.left_q_i   (left_q_i   ),
	.middle_q_i (middle_q_i ),
	.right_q_i  (right_q_i  ),
				
	.left_q_o   (left_q_o   ),
	.middle_q_o (middle_q_o ),
	.right_q_o  (right_q_o	)
);


s_box
#(
	.DATA_WIDTH(DATA_WIDTH),
	.Q_LEFT    (	1	  ),  
	.Q_MIDDLE  (    0     ),
	.Q_RIGHT   (    0     )
)
sbox2
(
	.clk_i	    (clk_i	    ),    
	.reset_ni   (reset_ni   ),
	.left_q_i   (left_q_i   ),
	.middle_q_i (middle_q_i ),
	.right_q_i  (right_q_i  ),
				
	.left_q_o   (left_q_o   ),
	.middle_q_o (middle_q_o ),
	.right_q_o  (right_q_o	)
);


s_box
#(
	.DATA_WIDTH(DATA_WIDTH),
	.Q_LEFT    (	0	  ),  
	.Q_MIDDLE  (    1     ),
	.Q_RIGHT   (    1     )
)
sbox3
(
	.clk_i	    (clk_i	    ),    
	.reset_ni   (reset_ni   ),
	.left_q_i   (left_q_i   ),
	.middle_q_i (middle_q_i ),
	.right_q_i  (right_q_i  ),
				
	.left_q_o   (left_q_o   ),
	.middle_q_o (middle_q_o ),
	.right_q_o  (right_q_o	)
);


s_box
#(
	.DATA_WIDTH(DATA_WIDTH),
	.Q_LEFT    (	1	  ),  
	.Q_MIDDLE  (    1     ),
	.Q_RIGHT   (    0     )
)
sbox4
(
	.clk_i	    (clk_i	    ),    
	.reset_ni   (reset_ni   ),
	.left_q_i   (left_q_i   ),
	.middle_q_i (middle_q_i ),
	.right_q_i  (right_q_i  ),
				
	.left_q_o   (left_q_o   ),
	.middle_q_o (middle_q_o ),
	.right_q_o  (right_q_o	)
);


always begin
	#15 clk_i <= ~clk_i;
end

initial begin
	clk_i = 'b0;
	reset_ni = 'b1;
	left_q_i  ='bx;
    middle_q_i='bx;
    right_q_i ='bx;
	@(posedge clk_i)
	reset_ni = 'b0;
	left_q_i   ='d1;
    middle_q_i ='d1;
    right_q_i  ='d1;
	@(posedge clk_i)
	reset_ni = 'b1;
	@(posedge clk_i)
	left_q_i   ='d2;
    middle_q_i ='d2;
    right_q_i  ='d2;
end
endmodule